const system = require("../../../system");
// const Dao = require("../../dao.base");
const ServiceBase = require("../../sve.base");
const uuidv4 = require('uuid/v4');
class OrderInfoService extends ServiceBase {
  constructor() {
    super("dbcorder", ServiceBase.getDaoName(OrderInfoService));
    this.orderReceiptVoucherDao = system.getObject("db.dbcpay.orderreceiptvoucherDao");
    this.orderRefundVoucherDao = system.getObject("db.dbcpay.orderrefundvoucherDao");
    this.moneyJourneyDao = system.getObject("db.dbcpay.moneyjourneyDao");
    this.pushlogSve = system.getObject("service.common.pushlogSve");
  }
  /*
   * 订单验证
  */
  async isOrderVerify(pobj) {
    var actionBody = pobj.actionBody;
    var price_list = actionBody.product_info.price_list;
    if (!price_list) {
      return system.getResult(null, "产品价格列表信息有误，20150");
    }
    var product_price = price_list.filter(f => f.id == actionBody.id);
    if (!product_price || product_price.length == 0) {
      return system.getResult(null, "产品价格信息有误，20180");
    }
    var totalSum = Number(product_price[0].price) * Number(actionBody.quantity);
    if (Number(actionBody.totalSum) < totalSum) {
      return system.getResult(null, "订单金额有误，20210");
    }
    pobj.actionBody.product_info.price_item = product_price[0];
    return system.getResultSuccess();
  }
  async createOrder(pobj, orderNo, t) {
    var actionBody = pobj.actionBody;
    var channelOrder = actionBody.channelOrder;
    var price_item = actionBody.product_info.price_item;
    var totalServiceCharge = Number(price_item.service_charge || 0) * Number(actionBody.quantity);
    var totalPublicExpense = Number(price_item.public_expense || 0) * Number(actionBody.quantity);
    var totalProfitSum = Number(actionBody.totalSum) - (Number(price_item.supply_price || 0) * Number(actionBody.quantity));
    var pfProfitSum = (100 - Number(price_item.channel_profit_rate)) / 100 * totalProfitSum;
    if (pfProfitSum < 0) {
      pfProfitSum = 0;
    }
    var channelProfitSum = Number(totalProfitSum) - Number(pfProfitSum);
    var payTime = channelOrder.payTime || new Date();
    var params = {
      uapp_id: pobj.appInfo.uapp_id,//int(11)       //                                                                                                                                                                                     
      orderNo: orderNo,//varchar(64)   //订单号                                                                                                                                                                                        
      channelServiceNo: channelOrder.channelServiceNo || "",//varchar(64)   //渠道服务单号                                                                                                                                                                               
      channelOrderNo: channelOrder.channelOrderNo || "",//varchar(1024) //渠道订单号列表，多个以,隔开                                                                                                                                                         
      channelUserId: pobj.userInfo.channel_userId,//varchar(64)   //                                                                                                                                                                                                 
      ownerUserId: pobj.userInfo.channel_userId,//varchar(20)   //                                                                                                                                                                                                 
      needNo: channelOrder.needNo || "",//varchar(64)   //需求单号                                                                                                                                                                                     
      needNoOrderNo: channelOrder.needNoOrderNo || "",//varchar(64)   //需求订单号                                                                                                                                                                                  
      payTime: channelOrder.orderStatus && channelOrder.orderStatus == 1 ? payTime : null,//datetime      //                                                                                                                                                                                                 
      quantity: actionBody.quantity,//int(11)       //项目订单数量（即服务项目的倍数，默认值为1）                                                                                                                                 
      serviceQuantity: 0,//int(11)       //项目订单交付数量（即与项目订单数量相对应）                                                                                                                                                                                                   
      orderStatus: channelOrder.orderStatus || 1,//int          //订单状态: 1: 待付款, 2: 已付款, 4: 待服务, 8: 已完成
      totalSum: actionBody.totalSum,//decimal(12,2) //订单总额(产品价格×优惠费率×订单件数)                                                                                                                                           
      payTotalSum: actionBody.payTotalSum,//decimal(12,2) //                                                                                                                                                                                                 
      refundSum: 0,//decimal(12,2) //退款总额
      totalServiceCharge: totalServiceCharge,//decimal(12,2) //服务费总额（产品配置的服务费*订单件数）                                                                                                                                       
      totalPublicExpense: totalPublicExpense,//decimal(12,2) //官费总额（产品配置的官费*订单件数）   
      totalProfitSum: totalProfitSum,//decimal(12,2) //订单毛利润总额（产品价格-供货价格））  
      pfProfitSum: pfProfitSum,//decimal(12,2) //订单平台毛利润总额（(100-渠道利润分成比例)/100 * 订单毛利润总额）                                                                                                         
      channelProfitSum: channelProfitSum,//decimal(12,2) //订单渠道分成毛利润总额(订单毛利润总额-平台毛利润总额)  

      totalDiscounts: 0,//decimal(12,2) //优惠总额（（服务费总额+官费总额）-订单总额(产品价格×优惠费率×订单件数)>0则有优惠额度）暂时没有用到                                                                                                                                                    
      pfSettleProfit: 0,//int(11)       //平台结算渠道利润,0否，1是                                                                                                                                                             
      invoiceApplyStatus: "00",//varchar(10)   //发票状态:00: 未申请, 10: 已申请,20:已开票                                                                                                                                           
      notes: ,//varchar(255)  //                                                                                                                                                                                                 
      opNotes: ,//varchar(255)  //操作备注       
    };
    var order = await this.dao.create(params, t);//创建订单
    var orderProductObj = {
      uapp_id: pobj.appInfo.uapp_id,
      sourceOrderNo: orderNo,   // 来源单号
      productType_id: price_item.productType_id,//产品类型Id
      pathCode: price_item.path_code,//产品路径
      itemCode: price_item.item_code,//产品编码
      itemName: price_item.item_name,//产品名称                                               
      channelItemCode: price_item.channel_item_code,//  渠道产品编码                                                 
      channelItemName: price_item.channel_item_name,//  渠道产品名称 
      serviceItemCode: price_item.service_item_code,//  服务商产品编码
      picUrl: price_item.pic_url,//  产品图片地址      
      proPrice: actionBody.totalSum,    //  产品总价格    
      quantity: actionBody.quantity,//  订单数量（即产品的倍数，默认值为1）   
      opPayType: "00",//  操作付款类型:00: 创建订单, 10: 补单
      serviceItemSnapshot: JSON.stringify(actionBody.product_info),      //产品快照  
    };
    var orderProduct = await this.orderProductDao.create(orderProductObj, t);//订单产品  
    if (channelOrder.orderStatus && channelOrder.orderStatus == 2) {//订单状态: 1: 待付款, 2: 已付款, 4: 待服务, 8: 已完成
      var moneyObj = {
        uapp_id: pobj.appInfo.uapp_id,
        sourceOrderNo: orderNo,   // 来源单号
        channelUserId: pobj.userInfo.channel_userId,
        ownerUserId: pobj.userInfo.channel_userId,
        accountType: "other",//帐户类型（ 支付类型）:"cash": "现金", "bank": "银行" ,"wx":"微信","alipay":"支付宝","other":"其它"
        directionType: "sr",//凭单类型,"sr": "收","zc": "支"
        voucherDate: payTime,//凭单时间
        recvAmount: channelOrder.payTotalSum,//收总额
        sourceType: "orderinfo",//来源类型 "orderinfo": "订单","expensevoucher": "费用单"
        auditStatus: "tg",//审核状态"dsh": "待审核", "btg": "不通过", "tg": "通过"
      };
      var moneyJourney = await this.moneyJourneyDao.create(moneyObj, t);
      var orderReceiptVoucherObj = {
        uapp_id: pobj.appInfo.uapp_id,
        sourceOrderNo: orderNo,   // 来源单号
        accountType: "other",
        payDate: payTime,//支付时间
        totalSum: channelOrder.payTotalSum,//订单总额
        auditStatus: "tg"
        // payOrderNo: DataTypes.STRING, //支付凭证流水单号，如：微信支付凭证单号
        // buyerOpenId: DataTypes.STRING,//用户在支付商户appid下的唯一标识或买家在支付宝的用户id
        // passTradeNo: DataTypes.STRING,//通道的统一订单号
        // buyerAliLogonId: DataTypes.STRING,//买家支付宝账号
        // certifyFileUrl: order.orderNo, //支付证明文件Url
        // wxPayOrderCode: order.orderNo,//业务微信支付订单号
        // aliPayOrderCode: order.orderNo,//业务支付宝支付订单号
        // busPayOrderCode: order.orderNo,//业务支付订单号

      };
      var orderReceiptVoucher = await this.orderReceiptVoucherDao.create(orderReceiptVoucherObj, t);
    }
  }
  /**
   * 创建商标订单
   * @param {*} pobj 
   * {
          "actionType": "addOrder",
          "actionBody": {
            "channelItemCode":"zzsbzc",
            "id":1,
            "quantity":1,
            "totalSum":300,
            "channelOrder":null,
            "deliveryData":null
          }
      }
   */
  async createTmOrder(pobj) {
    var verifyResult = await this.isOrderVerify(pobj);
    if (verifyResult.status != 0) {
      return verifyResult;
    }
    var self = this;
    return await self.db.transaction(async function (t) {
      var orderNo = await self.getBusUid("tm" + pobj.appInfo.uapp_id);
      var item = self.createOrder(pobj, orderNo, t);
      return system.getResultSuccess({
        orderNo: orderNo,
        channelServiceNo: pobj.actionBody.channelOrder.channelServiceNo || orderNo,
        channelParams: opobj.actionBody.channelOrder.channelParams || ""
      });
    });

  }
  /**
   * 创建其他订单
   * @param {*} pobj 
   */
  async createOtherOrder(pobj) {
    var verifyResult = await this.isOrderVerify(pobj);
    if (verifyResult.status != 0) {
      return verifyResult;
    }
    var self = this;
    return await self.db.transaction(async function (t) {
      var orderNo = await self.getBusUid("ot" + pobj.appInfo.uapp_id);
      var item = self.createOrder(pobj, orderNo, t);
      return system.getResultSuccess({
        orderNo: orderNo,
        channelServiceNo: pobj.actionBody.channelOrder.channelServiceNo || orderNo,
        channelParams: opobj.actionBody.channelOrder.channelParams || ""
      });
    });

  }

  async createChannelSource(pobj) {//创建渠道来源订单
    var orderNo = await self.getBusUid("oc" + pobj.appInfo.uapp_id);
    // var beforeOrder = await this.dao.model.findOne({
    //   where: { channelServiceNo: channelOrder.channelServiceNo, uapp_id: app.uAppId },
    //   raw: true
    // });
    // if (beforeOrder) {//订单已存在
    //   return system.getResultFail(2, "订单已存在", beforeOrder.orderNo);
    // }
  }
  /**
   * 数据推送
   */
  // async pushData(requestid, params, url, req) {
  //   var rc = system.getObject("util.execClient");
  //   var obj = params;
  //   var rtn = null;
  //   try {
  //     rtn = await rc.execPushDataPost(obj, url, req.headers["token"], req.headers["request-id"]);
  //     var returnType = "0";
  //     if (rtn.stdout) {
  //       var j = JSON.parse(rtn.stdout);
  //       if (j && j.status == 0) {
  //         returnType = "1";
  //       }
  //     }
  //     this.pushlogSve.createDb({
  //       appid: req.app.id,
  //       requestId: requestid,
  //       op: url,
  //       content: JSON.stringify(obj),
  //       resultInfo: JSON.stringify(rtn),
  //       clientIp: req.clientIp,
  //       returnType: returnType,
  //       agent: req.uagent,
  //       opTitle: "数据推送-创建订单",
  //     });
  //   } catch (e) {
  //     this.pushlogSve.createDb({
  //       appid: req.app.id,
  //       requestId: requestid,
  //       op: url,
  //       content: JSON.stringify(obj),
  //       resultInfo: JSON.stringify(e.stack),
  //       clientIp: req.clientIp,
  //       returnType: '0',
  //       agent: req.uagent,
  //       opTitle: "数据推送-创建订单",
  //     });
  //   }
  // }
  // async findAndCountAll(obj, req) {
  //   var app = req.app;
  //   if (!app || !app.uAppId) {
  //     return system.getResult(null, "渠道信息有误");
  //   }
  //   obj["search"]["uapp_id"] = app.uAppId;
  //   const result = await this.dao.findAndCountAll(obj);
  //   return system.getResultSuccess(result);
  // }

  // async getOrderDetailByOrderNo(obj, req) {
  //   var orderNo = obj.orderNo;
  //   if (!orderNo) {
  //     return system.getResult(null, "订单编号不能为空");
  //   }
  //   var app = req.app;
  //   if (!app || !app.uAppId) {
  //     return system.getResult(null, "渠道信息有误");
  //   }
  //   var order = await this.dao.model.findOne({
  //     where: { orderNo: orderNo, uapp_id: app.uAppId },
  //     attributes: ["orderNo", "channelServiceNo", "channelOrderNo", "channelUserId", "ownerUserId", "needNo",
  //       "payTime", "quantity", "serviceQuantity", "orderPayStatus", "orderPayStatusName", "totalSum", "payTotalSum",
  //       "refundSum", "created_at", "opNotes", "notes"
  //     ],
  //     raw: true
  //   });
  //   if (order && order.orderNo) {
  //     var orderproducts = await this.orderProductDao.model.findAll({
  //       where: { sourceOrderNo: order.orderNo },
  //       attributes: ["sourceOrderNo", "itemCode", "itemName", "channelItemCode", "channelItemName",
  //         "serviceItemCode", "picUrl", "proPrice", "quantity", "opPayType", "serviceItemSnapshot",
  //         "created_at"
  //       ],
  //       raw: true
  //     });
  //     order["orderproducts"] = orderproducts;
  //     var receptvouchers = await this.orderReceiptVoucherDao.model.findAll({
  //       where: { sourceOrderNo: order.orderNo },
  //       attributes: [
  //         "sourceOrderNo", "accountType", "accountTypeName", "payDate", "totalSum", "payOrderNo", "buyerOpenId",
  //         "passTradeNo", "buyerAliLogonId", "certifyFileUrl", "wxPayOrderCode", "aliPayOrderCode", "busPayOrderCode",
  //         "auditStatusName", "auditStatus"
  //       ],
  //       raw: true
  //     });
  //     order["receptvouchers"] = receptvouchers;
  //     var refundvouchers = await this.orderRefundVoucherDao.model.findAll({
  //       where: { sourceOrderNo: order.orderNo },
  //       attributes: [
  //         "busPayOrderCode",
  //         "sourceOrderNo", "accountType", "accountTypeName", "payDate", "totalSum", "payOrderNo", "buyerOpenId",
  //         "passTradeNo", "certifyFileUrl", "auditStatusName", "auditStatus"
  //       ],
  //       raw: true
  //     });
  //     order["refundvouchers"] = refundvouchers;
  //   }
  //   return system.getResultSuccess(order);
  // }
}
module.exports = OrderInfoService;